@@ -362,6 +362,5 @@ class DG1062(VisaInstrument):
             channels.append(channel)
             self.add_submodule(ch_name, channel)
 
-        channels.lock()
-        self.add_submodule("channels", channels)
+        self.add_submodule("channels", channels.to_channel_tuple())
         self.connect_message()
